// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.key.polytomous;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.IManagedForm;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.description.KeyStatement;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ISelectable;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;

/**
 * @author n.hoffmann
 * @created Apr 13, 2011
 * @version 1.0
 */
public class PolytomousKeyListItem extends AbstractCdmFormElement implements
		IFormPart, ISelectable {

	private boolean dirty;
	private final PolytomousKeyNode entity;

	private SelectionListener linkSelectionListener;

	/**
	 * @param formFactory
	 * @param layoutComposite
	 */
	protected PolytomousKeyListItem(CdmFormFactory formFactory,
			Composite layoutComposite, PolytomousKeyNode entity) {
		super(formFactory, layoutComposite);

		// layoutComposite.setBackground(Display.getDefault().getSystemColor(
		// SWT.COLOR_CYAN));

		this.entity = (PolytomousKeyNode) HibernateProxyHelper.deproxy(entity);

		Label label_nodeNumber = new Label(getLayoutComposite(), SWT.NULL);
		label_nodeNumber.setText(getItemNumber());

		Label label_question = new Label(getLayoutComposite(), SWT.NULL);
		label_question.setText(getItemQuestion());

		Label label_statement = new Label(getLayoutComposite(), SWT.NULL);
		label_statement.setText(getItemStatement());
		label_statement.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

		Link link = new Link(getLayoutComposite(), SWT.NONE);
		link.setText("<a>" + getItemLink() + "</a>");
		link.setData(getItemLinkData());
		link.addSelectionListener(getLinkSelectionListener());

		// Label label_link = new Label(getLayoutComposite(), SWT.NULL);
		// label_link.setText(link);

	}

	/**
	 * @return
	 */
	private SelectionListener getLinkSelectionListener() {
		if (linkSelectionListener == null) {
			linkSelectionListener = new SelectionAdapter() {
				/*
				 * (non-Javadoc)
				 * 
				 * @see
				 * org.eclipse.swt.events.SelectionAdapter#widgetSelected(org
				 * .eclipse.swt.events.SelectionEvent)
				 */
				@Override
				public void widgetSelected(SelectionEvent e) {
					Object data = e.widget.getData();

					if (data instanceof Taxon) {
						try {
							EditorUtil.openTaxonBase(((Taxon) data).getUuid());
						} catch (PartInitException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						}
					}
				}
			};
		}
		return linkSelectionListener;
	}

	private String getItemNumber() {
		if (isParentRoot()) {
			return "root";
		} else {
			PolytomousKeyNode parent = getParent();
			String itemNumber = parent.getNodeNumber() != null ? parent
					.getNodeNumber().toString() : "NaN";

			int index = parent.getChildren().indexOf(entity);

			for (int i = 0; i < index; i++) {
				itemNumber += "'";
			}

			return itemNumber;
		}
	}

	private String getItemQuestion() {
		if (isParentRoot()) {
			return "";
		} else {
			KeyStatement question = getParent().getQuestion();
			return question != null ? question.getLabelText(CdmStore
					.getDefaultLanguage()) : "";
		}

	}

	private String getItemStatement() {
		KeyStatement statement = entity.getStatement();
		return statement != null ? CdmUtils.Nz(statement.getLabelText(CdmStore
				.getDefaultLanguage())) : "No statement";
	}

	private String getItemLink() {
		String taxonString = entity.getTaxon() != null ? entity.getTaxon()
				.getName().getTitleCache() : "Taxon empty";

		return entity.getChildren().isEmpty() ? taxonString : entity
				.getNodeNumber().toString();
	}

	/**
	 * @return
	 */
	private Object getItemLinkData() {
		return entity.getChildren().isEmpty() ? entity.getTaxon() : entity
				.getChildAt(0);
	}

	private PolytomousKeyNode getParent() {
		return entity.getParent();
	}

	private boolean isParentRoot() {
		return getParent() == null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.forms.IFormPart#initialize(org.eclipse.ui.forms.IManagedForm
	 * )
	 */
	@Override
	public void initialize(IManagedForm form) {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IFormPart#isDirty()
	 */
	@Override
	public boolean isDirty() {
		return dirty;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IFormPart#commit(boolean)
	 */
	@Override
	public void commit(boolean onSave) {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IFormPart#setFormInput(java.lang.Object)
	 */
	@Override
	public boolean setFormInput(Object input) {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IFormPart#setFocus()
	 */
	@Override
	public void setFocus() {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IFormPart#isStale()
	 */
	@Override
	public boolean isStale() {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IFormPart#refresh()
	 */
	@Override
	public void refresh() {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IFormPart#dispose()
	 */
	@Override
	public void dispose() {
		// TODO Auto-generated method stub

	}

	/**
	 * @return the entity
	 */
	public PolytomousKeyNode getEntity() {
		return entity;
	}
	
	@Override
	public void setSelected(boolean selected) {
		setBackground(selected ? SELECTED : getPersistentBackground());
	}

}
