// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.handler;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.Page;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.editor.name.operation.CreateSynonymInExistingHomotypicalGroupOperation;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.parser.ParseHandler;

/**
 * <p>CreateSynonymInHomotypicalGroupHandler class.</p>
 *
 * @author n.hoffmann
 * @created 21.04.2009
 * @version 1.0
 */
public class CreateSynonymInHomotypicalGroupHandler extends AbstractHandler
		implements IHandler {
	private static final Logger logger = Logger
			.getLogger(CreateSynonymInHomotypicalGroupHandler.class);

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	/** {@inheritDoc} */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		TaxonNameEditor editor = (TaxonNameEditor) EditorUtil.getActiveEditorPage(
				Page.NAME);
		ISelection menuSelection = HandlerUtil.getCurrentSelection(event);
		
		Object selecteObject = ((StructuredSelection) menuSelection).getFirstElement();
		
		if(! (selecteObject instanceof Synonym)){
			return null;
		}
		
		
		Synonym synonym = (Synonym) selecteObject;
		
		HomotypicalGroup group = synonym.getHomotypicGroup();
		
		TaxonNameBase newSynonymName = ParseHandler.createEmptyName();
		
		AbstractPostOperation operation;
		
		try {
			operation = new CreateSynonymInExistingHomotypicalGroupOperation(event.getCommand().getName(), editor.getUndoContext(),
								editor.getTaxon(), group, newSynonymName, editor);
			EditorUtil.executeOperation(operation);
		} catch (NotDefinedException e) {
			logger.warn("Command name not set");
		}
		
		return null;
	}
}
