/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.handler;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.commands.common.NotDefinedException;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.Page;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.editor.name.operation.DeleteMisapplicationOperation;
import eu.etaxonomy.taxeditor.editor.name.operation.DeleteSynonymOperation;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;

/**
 * <p>DeleteTaxonBaseHandler class.</p>
 *
 * @author n.hoffmann
 * @created 21.04.2009
 * @version 1.0
 */
public class DeleteTaxonBaseHandler extends AbstractHandler implements IHandler {

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	/** {@inheritDoc} */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		TaxonNameEditor editor = (TaxonNameEditor) EditorUtil.getActiveEditorPage(Page.NAME);
				
		Object selectedElement = EditorUtil.getSelection(event).getFirstElement();
		
		doExecute(event, editor, selectedElement);
		
		return null;
	}
	
	/**
	 * @param editor
	 * @param selectedElement
	 * @param operation
	 * @param commandName
	 */
	protected void doExecute(ExecutionEvent event, TaxonNameEditor editor, Object selectedElement) {
		AbstractPostOperation operation = null;
		String commandName = null;		

		try {
			commandName = event.getCommand().getName();
		} catch (NotDefinedException e) {
			EditorUtil.error(getClass(), e);
		}
		
		// synonym
		if(selectedElement instanceof Synonym){
			operation = new DeleteSynonymOperation(commandName, editor.getUndoContext(), editor.getTaxon(), (Synonym) selectedElement, editor);
		}
		// misapplication
		else if(selectedElement instanceof Taxon && ((Taxon) selectedElement).isMisapplication()){
			operation = new DeleteMisapplicationOperation(commandName, editor.getUndoContext(), editor
					.getTaxon(), (Taxon) selectedElement, editor);
		}
		else {
			throw new IllegalArgumentException("Element has to be Synonym, Misapplication or Concept");
		}
				
		EditorUtil.executeOperation(operation);
	}

}
