package eu.etaxonomy.taxeditor.editor.view.derivate;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.CompoundContributionItem;
import org.eclipse.ui.handlers.IHandlerService;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeCacheStrategy;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.model.occurrence.DerivationEvent;
import eu.etaxonomy.cdm.model.occurrence.DerivationEventType;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class CreateDerivateContextMenu extends CompoundContributionItem {

    private final IHandlerService handlerService = (IHandlerService) EditorUtil.getService(IHandlerService.class);

    /* (non-Javadoc)
     * @see org.eclipse.ui.actions.CompoundContributionItem#getContributionItems()
     */
    @Override
    protected IContributionItem[] getContributionItems() {
        IContributionItem[] contributionItems = new IContributionItem[] {
                new ContributionItem() {
                    @Override
                    public void fill(Menu menu, int index) {
                        final IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
                        final ISelection selection = window.getActivePage().getSelection();
                        if(selection instanceof TreeSelection){
                            final Object selectedElement = ((TreeSelection) selection).getFirstElement();
                            //context menu for FieldUnit
                            if(selectedElement instanceof FieldUnit){
                                MenuItem createSpecimen = new MenuItem(menu, SWT.None);
                                createSpecimen.setText("Specimen");
                                createSpecimen.addSelectionListener(new SelectionListener() {

                                    @Override
                                    public void widgetSelected(SelectionEvent e) {
                                        IOccurrenceService occurrenceService = CdmStore.getCurrentApplicationConfiguration().getOccurrenceService();
                                        SpecimenOrObservationBase fieldUnit = occurrenceService.load(((FieldUnit) selectedElement).getUuid());

                                        DerivedUnit specimen = DerivedUnit.NewInstance(SpecimenOrObservationType.PreservedSpecimen);
                                        fieldUnit.addDerivationEvent(DerivationEvent.NewSimpleInstance(fieldUnit, specimen, DerivationEventType.GATHERING_IN_SITU()));
                                        specimen.setCacheStrategy(new DerivedUnitFacadeCacheStrategy());
//                                        specimen.getTitleCache(); //update title cache
                                        IWorkbenchPart activePart = AbstractUtility.getActivePart();
                                        occurrenceService.saveOrUpdate(fieldUnit);
                                        if(activePart instanceof DerivateView){
                                            DerivateView derivateView = (DerivateView)activePart;
                                            derivateView.viewer.refresh();
                                        }
//                                        DerivedUnit specimen = DerivedUnit.NewInstance(SpecimenOrObservationType.PreservedSpecimen);
//                                        fieldUnit.addDerivationEvent(DerivationEvent.NewSimpleInstance(fieldUnit, specimen, DerivationEventType.GATHERING_IN_SITU()));
//                                        specimen.setCacheStrategy(new DerivedUnitFacadeCacheStrategy());
//                                        specimen.getTitleCache(); //update title cache
//                                        IEditorPart activeEditor = AbstractUtility.getActiveEditor();
//                                        IWorkbenchPart activePart = AbstractUtility.getActivePart();
//                                        ConversationHolder conversation = CdmStore.createConversation();
//                                        CdmStore.getService(IOccurrenceService.class).saveOrUpdate(fieldUnit);
//                                        conversation.commit();
//                                        conversation.close();
//                                        if(activePart instanceof DerivateView){
//                                            DerivateView derivateView = (DerivateView)activePart;
//                                            derivateView.viewer.refresh();
//                                        }
                                    }

                                    @Override
                                    public void widgetDefaultSelected(SelectionEvent e) {
                                        // TODO Auto-generated method stub

                                    }
                                });
                                MenuItem createLivingPlantPhoto = new MenuItem(menu, SWT.None);
                                createLivingPlantPhoto.setText("Living Plant Photo");
                            }
                            //context menu for Specimen
                            else if(selectedElement instanceof DerivedUnit && ((DerivedUnit)selectedElement).getRecordBasis()==SpecimenOrObservationType.PreservedSpecimen){
                                MenuItem createTissueSample = new MenuItem(menu, SWT.None);
                                createTissueSample.setText("Tissue Sample");
                                MenuItem createDNASample = new MenuItem(menu, SWT.None);
                                createDNASample.setText("DNA Sample");
                                MenuItem createSpecimenScan = new MenuItem(menu, SWT.None);
                                createSpecimenScan.setText("Specimen Scan");
                                MenuItem createArtwork = new MenuItem(menu, SWT.None);
                                createArtwork.setText("Artwork");
                                MenuItem createLivingPlantPhoto = new MenuItem(menu, SWT.None);
                                createLivingPlantPhoto.setText("Living Plant Photo");
                            }
                            else{
                                MenuItem menuItem = new MenuItem(menu, SWT.None);
                                menuItem.setText("No child derivates");
                                menuItem.setEnabled(false);
                            }
                        }
                    }
                }
        };
        return contributionItems;
    }

}
