package eu.etaxonomy.taxeditor.editor.view.derivate;

import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IWorkbenchPart;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.view.AbstractCdmEditorViewPart;

/**
 * Displays the derivate hierarchy of all derivates belonging to a taxon.
 *
 */
public class DerivateView extends AbstractCdmEditorViewPart implements IPartContentHasDetails, IPartContentHasSupplementalData {

	/** Constant <code>ID="eu.etaxonomy.taxeditor.editor.view.desc"{trunked}</code> */
	public static final String ID = "eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView";

	protected TreeViewer viewer;

	@Override
	public void createViewer(Composite parent) {
		viewer = new TreeViewer(new Tree(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION));
		viewer.setContentProvider(new DerivateContentProvider());
		viewer.setLabelProvider(new DerivateLabelProvider());
		viewer.setAutoExpandLevel(2);
        // Propagate selection from viewer
        getSite().setSelectionProvider(viewer);

        //create context menu
        MenuManager menuManager = new MenuManager();

        getSite().registerContextMenu(menuManager, viewer);

        Control control = viewer.getControl();
        Menu menu = menuManager.createContextMenu(control);

        control.setMenu(menu);
	}

	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
	    if(AbstractUtility.getActiveEditor()==null || !(AbstractUtility.getActiveEditor() instanceof BulkEditor)){
	        showEmptyPage();
	        return;
	    }
	    else if(part instanceof BulkEditor){
	        this.part = part;

	        getViewer().setInput(selection);

	        showViewer();
	        return;
	    }
	}

	@Override
	public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public Viewer getViewer() {
		return viewer;
	}


	@Override
    public boolean postOperation(CdmBase objectAffectedByOperation) {
		return super.postOperation(objectAffectedByOperation);
	}


}
