// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import org.eclipse.core.commands.ExecutionException;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.store.operations.AbstractTaxeditorOperationTestBase;

/**
 * @author n.hoffmann
 * @created 08.04.2009
 * @version 1.0
 */
public class MoveTaxonOperationTest extends AbstractTaxeditorOperationTestBase {

	private static Taxon oldParentTaxon;
	private static Taxon newParentTaxon;
	private static Classification tree;
	private static TaxonNode oldParentTaxonNode;
	private static TaxonNode newParentTaxonNode;
	private static TaxonNode taxonNode;
	
	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {

		
		taxon = Taxon.NewInstance(null, null);
		
		
		
		oldParentTaxon = Taxon.NewInstance(null, null);
		newParentTaxon = Taxon.NewInstance(null, null);
		
		tree = Classification.NewInstance(null);
		oldParentTaxonNode = tree.addChildTaxon(oldParentTaxon, null, null);
		newParentTaxonNode = tree.addChildTaxon(newParentTaxon, null, null);
		
		taxonNode = oldParentTaxonNode.addChildTaxon(taxon, null, null);
		
		operation = null;//new MoveTaxonOperation("Move Taxon To Different Parent", undoContext, taxonNode, newParentTaxonNode, postOperation);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.navigation.operation.MoveTaxonOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testExecuteIProgressMonitorIAdaptable() throws ExecutionException {
		operation.execute(monitor, info);
		
		Assert.assertEquals(newParentTaxon ,taxon.getTaxonomicParent());
		Assert.assertEquals(taxon, newParentTaxon.getTaxonomicChildren().toArray(new Taxon[0])[0]);
		Assert.assertTrue(oldParentTaxon.getTaxonomicChildrenCount() == 0);
	}


	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.navigation.operation.MoveTaxonOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testUndoIProgressMonitorIAdaptable() throws ExecutionException {
		operation.undo(monitor, info);
		
		Assert.assertEquals(oldParentTaxon ,taxon.getTaxonomicParent());
		Assert.assertTrue(newParentTaxon.getTaxonomicChildren().size() == 0);
		Assert.assertEquals(taxon, oldParentTaxon.getTaxonomicChildren().toArray(new Taxon[0])[0]);
	}
	
	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.navigation.operation.MoveTaxonOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testRedoIProgressMonitorIAdaptable() throws ExecutionException {
		operation.redo(monitor, info);
		
		Assert.assertEquals(newParentTaxon ,taxon.getTaxonomicParent());
		Assert.assertEquals(taxon, newParentTaxon.getTaxonomicChildren().toArray(new Taxon[0])[0]);
		Assert.assertTrue(oldParentTaxon.getTaxonomicChildrenCount() == 0);
	}
}
